From d1365b84ccf7c1028fca5f7056781b2fc95093a1 Mon Sep 17 00:00:00 2001
From: amit <amitlangote09@gmail.com>
Date: Mon, 2 Jul 2018 16:29:19 +0900
Subject: [PATCH v1] Set correct memory context in
 hypothetical_dense_rank_final

One of the memory context that it caches in OSAPerGroupState, which
is passed to it on every call, is parented by a memory context of
wrong life time.  Fix that by switching to a per-query context before
creating the aforementioned context.
---
 src/backend/utils/adt/orderedsetaggs.c   | 8 ++++++++
 src/test/regress/expected/aggregates.out | 9 +++++++++
 src/test/regress/sql/aggregates.sql      | 3 +++
 3 files changed, 20 insertions(+)

diff --git a/src/backend/utils/adt/orderedsetaggs.c b/src/backend/utils/adt/orderedsetaggs.c
index ed36851fdd..a758748cc7 100644
--- a/src/backend/utils/adt/orderedsetaggs.c
+++ b/src/backend/utils/adt/orderedsetaggs.c
@@ -1310,7 +1310,15 @@ hypothetical_dense_rank_final(PG_FUNCTION_ARGS)
 	osastate = (OSAPerGroupState *) PG_GETARG_POINTER(0);
 	econtext = osastate->qstate->econtext;
 	if (!econtext)
+	{
+		MemoryContext qcontext = osastate->qstate->qcontext;
+		MemoryContext oldcontext;
+
+		/* Make sure to we create econtext under correct parent context. */
+		oldcontext = MemoryContextSwitchTo(qcontext);
 		osastate->qstate->econtext = econtext = CreateStandaloneExprContext();
+		MemoryContextSwitchTo(oldcontext);
+	}
 
 	/* Adjust nargs to be the number of direct (or aggregated) args */
 	if (nargs % 2 != 0)
diff --git a/src/test/regress/expected/aggregates.out b/src/test/regress/expected/aggregates.out
index 10d6bb6824..a120dd83f7 100644
--- a/src/test/regress/expected/aggregates.out
+++ b/src/test/regress/expected/aggregates.out
@@ -2092,3 +2092,12 @@ SELECT variance(unique1::int4), sum(unique1::int8) FROM tenk1;
 (1 row)
 
 ROLLBACK;
+-- test coverage for dense_rank
+SELECT dense_rank(x) WITHIN GROUP (ORDER BY x) FROM (VALUES (1),(1),(2),(2),(3),(3)) v(x) GROUP BY (x) ORDER BY 1;
+ dense_rank 
+------------
+          1
+          1
+          1
+(3 rows)
+
diff --git a/src/test/regress/sql/aggregates.sql b/src/test/regress/sql/aggregates.sql
index 1b6db50956..7e77467ecd 100644
--- a/src/test/regress/sql/aggregates.sql
+++ b/src/test/regress/sql/aggregates.sql
@@ -925,3 +925,6 @@ EXPLAIN (COSTS OFF)
 SELECT variance(unique1::int4), sum(unique1::int8) FROM tenk1;
 
 ROLLBACK;
+
+-- test coverage for dense_rank
+SELECT dense_rank(x) WITHIN GROUP (ORDER BY x) FROM (VALUES (1),(1),(2),(2),(3),(3)) v(x) GROUP BY (x) ORDER BY 1;
-- 
2.11.0

